@@ -1,6 +1,7 @@
-#include <assert.h>
 #include "reader.h"
 
+#include <assert.h>
+
 static void Reader_dealloc(hiredis_ReaderObject *self);
 static int Reader_init(hiredis_ReaderObject *self, PyObject *args, PyObject *kwds);
 static PyObject *Reader_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
@@ -155,6 +156,8 @@ static void Reader_dealloc(hiredis_ReaderObject *self) {
     redisReplyReaderFree(self->reader);
     if (self->encoding)
         free(self->encoding);
+    Py_XDECREF(self->protocolErrorClass);
+    Py_XDECREF(self->replyErrorClass);
 
     ((PyObject *)self)->ob_type->tp_free((PyObject*)self);
 }
@@ -238,14 +241,35 @@ static PyObject *Reader_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 }
 
 static PyObject *Reader_feed(hiredis_ReaderObject *self, PyObject *args) {
-    const char *str;
-    int len;
+    Py_buffer buf;
+    Py_ssize_t off = 0;
+    Py_ssize_t len = -1;
 
-    if (!PyArg_ParseTuple(args, "s#", &str, &len))
+    if (!PyArg_ParseTuple(args, "s*|nn", &buf, &off, &len)) {
         return NULL;
+    }
 
-    redisReplyReaderFeed(self->reader, str, len);
+    if (len == -1) {
+      len = buf.len - off;
+    }
+
+    if (off < 0 || len < 0) {
+      PyErr_SetString(PyExc_ValueError, "negative input");
+      goto error;
+    }
+
+    if ((off + len) > buf.len) {
+      PyErr_SetString(PyExc_ValueError, "input is larger than buffer size");
+      goto error;
+    }
+
+    redisReplyReaderFeed(self->reader, buf.buf + off, len);
+    PyBuffer_Release(&buf);
     Py_RETURN_NONE;
+
+error:
+    PyBuffer_Release(&buf);
+    return NULL;
 }
 
 static PyObject *Reader_gets(hiredis_ReaderObject *self) {
